@@ -13,7 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  *
  */
 
@@ -27,18 +28,20 @@
 #include "qpipe.h"
 
 #include <stdlib.h>
+#include <limits.h>
+
+// sorry, i've added this dependency for now, but it's easy enough to take
+//   with you if you want qpipe independent of qca
+#include "qca_safeobj.h"
 
 #ifdef Q_OS_WIN
 # include <QThread>
-# include <QTimer>
 # include <QMutex>
 # include <QWaitCondition>
 # include <QTextCodec>
 # include <QTextEncoder>
 # include <QTextDecoder>
 #else
-# include <QSocketNotifier>
-# include <QTimer>
 # include <QMutex>
 #endif
 
@@ -91,6 +94,34 @@ static void ignore_sigpipe()
 }
 #endif
 
+#ifdef Q_OS_WIN
+static int pipe_dword_cap_to_int(DWORD dw)
+{
+	if(sizeof(int) <= sizeof(DWORD))
+		return (int)((dw > INT_MAX) ? INT_MAX : dw);
+	else
+		return (int)dw;
+}
+
+static bool pipe_dword_overflows_int(DWORD dw)
+{
+	if(sizeof(int) <= sizeof(DWORD))
+		return (dw > INT_MAX) ? true : false;
+	else
+		return false;
+}
+#endif
+
+#ifdef Q_OS_UNIX
+static int pipe_size_t_cap_to_int(size_t size)
+{
+	if(sizeof(int) <= sizeof(size_t))
+		return (int)((size > INT_MAX) ? INT_MAX : size);
+	else // maybe silly..  can int ever be larger than size_t?
+		return (int)size;
+}
+#endif
+
 static bool pipe_set_blocking(Q_PIPE_ID pipe, bool b)
 {
 #ifdef Q_OS_WIN
@@ -147,12 +178,12 @@ static int pipe_read_avail(Q_PIPE_ID pipe)
 #ifdef Q_OS_WIN
 	DWORD i = 0;
 	if(PeekNamedPipe(pipe, 0, 0, 0, &i, 0))
-		bytesAvail = (int)i;
+		bytesAvail = pipe_dword_cap_to_int(i);
 #endif
 #ifdef Q_OS_UNIX
 	size_t nbytes = 0;
 	if(ioctl(pipe, FIONREAD, (char *)&nbytes) >= 0)
-		bytesAvail = *((int *)&nbytes);
+		bytesAvail = pipe_size_t_cap_to_int(nbytes);
 #endif
 	return bytesAvail;
 }
@@ -186,7 +217,7 @@ static int pipe_read(Q_PIPE_ID pipe, char *data, int max, bool *eof)
 		else
 			return -1;
 	}
-	bytesRead = (int)r;
+	bytesRead = (int)r; // safe to cast, since 'max' is signed
 #endif
 #ifdef Q_OS_UNIX
 	int r = 0;
@@ -218,7 +249,7 @@ static int pipe_write(Q_PIPE_ID pipe, const char *data, int size)
 	DWORD written;
 	if(!WriteFile(pipe, data, size, &written, 0))
 		return -1;
-	return (int)written;
+	return (int)written; // safe to cast, since 'size' is signed
 #endif
 #ifdef Q_OS_UNIX
 	ignore_sigpipe();
@@ -253,7 +284,7 @@ static int pipe_read_avail_console(Q_PIPE_ID pipe)
 {
 	DWORD count, i;
 	INPUT_RECORD *rec;
-	int n, total;
+	int n, icount, total;
 
 	// how many events are there?
 	if(!GetNumberOfConsoleInputEvents(pipe, &count))
@@ -272,11 +303,12 @@ static int pipe_read_avail_console(Q_PIPE_ID pipe)
 		free(rec);
 		return -1;
 	}
-	count = i; // use only the amount returned
+
+	icount = pipe_dword_cap_to_int(i); // process only the amount returned
 
 	// see which ones are normal keypress events
 	total = 0;
-	for(n = 0; n < (int)count; ++n)
+	for(n = 0; n < icount; ++n)
 	{
 		if(rec[n].EventType == KEY_EVENT)
 		{
@@ -389,11 +421,13 @@ static int pipe_write_console(Q_PIPE_ID pipe, const ushort *data, int size)
 		{
 			// convert number of bytes to number of unicode chars
 			i = (DWORD)QString::fromLocal8Bit(out.mid(0, i)).length();
+			if(pipe_dword_overflows_int(i))
+				return -1;
 		}
 	)
 	if(!ret)
 		return -1;
-	return (int)i;
+	return (int)i; // safe to cast since 'size' is signed
 }
 #endif
 
@@ -630,7 +664,7 @@ public:
 	Q_PIPE_ID pipe;
 	const char *data;
 	int size;
-	QTimer timer;
+	SafeTimer timer;
 	int total;
 
 	QPipeWriterPoll(Q_PIPE_ID id, QObject *parent = 0) : QPipeWriter(parent), timer(this)
@@ -776,6 +810,8 @@ protected:
 						result = -2;
 					else if(ret >= 1) // we got some data??  queue it
 						result = c;
+					else // will never happen
+						result = -2;
 
 					m.lock();
 					active = false;
@@ -801,7 +837,7 @@ class QPipeReaderPoll : public QPipeReader
 	Q_OBJECT
 public:
 	Q_PIPE_ID pipe;
-	QTimer timer;
+	SafeTimer timer;
 	bool consoleMode;
 
 	QPipeReaderPoll(Q_PIPE_ID id, QObject *parent = 0) : QPipeReader(parent), timer(this)
@@ -861,6 +897,8 @@ private:
 				result = -2;
 			else if(ret >= 1) // we got some data??  queue it
 				result = c;
+			else // will never happen
+				result = -2;
 
 			timer.stop();
 			emit canRead(result);
@@ -908,14 +946,14 @@ public:
 #ifdef Q_OS_WIN
 	bool atEnd, atError, forceNotify;
 	int readAhead;
-	QTimer *readTimer;
+	SafeTimer *readTimer;
 	QTextDecoder *dec;
 	bool consoleMode;
 	QPipeWriter *pipeWriter;
 	QPipeReader *pipeReader;
 #endif
 #ifdef Q_OS_UNIX
-	QSocketNotifier *sn_read, *sn_write;
+	SafeSocketNotifier *sn_read, *sn_write;
 #endif
 
 	Private(QPipeDevice *_q) : QObject(_q), q(_q), pipe(INVALID_Q_PIPE_ID)
@@ -1019,7 +1057,7 @@ public:
 			pipeReader->start();
 
 			// polling timer
-			readTimer = new QTimer(this);
+			readTimer = new SafeTimer(this);
 			connect(readTimer, SIGNAL(timeout()), SLOT(t_timeout()));
 
 			// updated: now that we have pipeReader, this no longer
@@ -1031,7 +1069,7 @@ public:
 			pipe_set_blocking(pipe, false);
 
 			// socket notifier
-			sn_read = new QSocketNotifier(pipe, QSocketNotifier::Read, this);
+			sn_read = new SafeSocketNotifier(pipe, QSocketNotifier::Read, this);
 			connect(sn_read, SIGNAL(activated(int)), SLOT(sn_read_activated(int)));
 #endif
 		}
@@ -1042,7 +1080,7 @@ public:
 			pipe_set_blocking(pipe, false);
 
 			// socket notifier
-			sn_write = new QSocketNotifier(pipe, QSocketNotifier::Write, this);
+			sn_write = new SafeSocketNotifier(pipe, QSocketNotifier::Write, this);
 			connect(sn_write, SIGNAL(activated(int)), SLOT(sn_write_activated(int)));
 			sn_write->setEnabled(false);
 #endif
@@ -1163,7 +1201,7 @@ int QPipeDevice::idAsInt() const
 #ifdef Q_OS_WIN
 	DWORD dw;
 	memcpy(&dw, &d->pipe, sizeof(DWORD));
-	return (int)dw;
+	return (int)dw; // FIXME? assumes handle value fits in signed int
 #endif
 #ifdef Q_OS_UNIX
 	return d->pipe;
@@ -1499,7 +1537,7 @@ public:
 	SecureArray sec_buf;
 	SecureArray sec_curWrite;
 #endif
-	QTimer readTrigger, writeTrigger, closeTrigger, writeErrorTrigger;
+	SafeTimer readTrigger, writeTrigger, closeTrigger, writeErrorTrigger;
 	bool canRead, activeWrite;
 	int lastWrite;
 	bool closeLater;
